
; Specs:
;
; - midi input (31250 bps)
; - 8 pulse outputs, pulse duration controlled by note-on velocity
; - 16 bit pulse counter resolution, full velocity->duration lookup table
;   per output
; - pulse counter clock: 25us (?)
; - 100% deterministic pulse duration
; - 8 hold outputs

;=============================================================================
;	Filename:	midima.asm
;=============================================================================
;	Author: 	Johannes Taelman
;	Company:	Johannes Taelman
;	Revision:	1.00
;	Date:		May 2005
;=============================================================================
;	Include Files:	p18f252.inc	V1.3
;=============================================================================


		list p=18f252		;list directive to define processor
		#include <p18f252.inc>	;processor specific definitions
;		#include <p18f252.inc>	;processor specific definitions
		#include "midi.asm"

; #define simulate 1 ; enable simulation
; #define heartbeat 1 ; enable heatbeats
;******************************************************************************
;Configuration bits
; The __CONFIG directive defines configuration data within the .ASM file.
; The labels following the directive are defined in the P18F252.INC file.
; The PIC18FXX2 Data Sheet explains the functions of the configuration bits.
; Change the following lines to suit your application.

	__CONFIG	_CONFIG1H, _OSCS_OFF_1H & _HSPLL_OSC_1H
	__CONFIG	_CONFIG2L, _BOR_ON_2L & _PWRT_ON_2L
	__CONFIG	_CONFIG2H, _WDT_OFF_2H
	__CONFIG	_CONFIG3H, _CCP2MX_OFF_3H
	__CONFIG	_CONFIG4L, _STVR_OFF_4L & _LVP_OFF_4L & _DEBUG_OFF_4L
	__CONFIG	_CONFIG5L, _CP0_OFF_5L & _CP1_OFF_5L & _CP2_OFF_5L & _CP3_OFF_5L 
	__CONFIG	_CONFIG5H, _CPB_OFF_5H & _CPD_OFF_5H
	__CONFIG	_CONFIG6L, _WRT0_OFF_6L & _WRT1_OFF_6L & _WRT2_OFF_6L & _WRT3_OFF_6L 
	__CONFIG	_CONFIG6H, _WRTC_OFF_6H & _WRTB_OFF_6H & _WRTD_OFF_6H
	__CONFIG	_CONFIG7L, _EBTR0_OFF_7L & _EBTR1_OFF_7L & _EBTR2_OFF_7L & _EBTR3_OFF_7L
	__CONFIG	_CONFIG7H, _EBTRB_OFF_7H



#define Timer1Out 	OutsA,	2;
#define Timer2Out 	OutsA,	1;
#define Timer3Out 	OutsA,	0;
#define Timer4Out 	OutsA,	3;
#define Timer5Out	OutsA,	4;
#define Timer6Out 	OutsA,	5;
#define Timer7Out 	OutsB,	7;
#define Timer8Out 	OutsB,	6;
#define Timer9Out 	OutsB,	5;
#define Timer10Out 	OutsB,	4;
#define Timer11Out 	OutsB,	3;
#define Timer12Out  OutsB,	2;
#define Timer13Out  OutsB,	1;
#define Timer14Out  OutsB,	0;
#define Timer15Out  OutsC,	5;
#define Timer16Out  OutsC,	4;

;----------------------------------------------------------------------------
; Variables
; memory map 
;----------------------------------------------------------------------------

		CBLOCK	0x000
		WREG_TEMP		;to save WREG during interrupt
		STATUS_TEMP		;to save STATUS during interrupt
		BSR_TEMP		;to save BSR during interrupt
		FSR0H_TEMP		;to save FSR0H during interrupt
		FSR0L_TEMP		;to save FSR0L during interrupt
		FSR0H_SHADOW		;to save FSR0H during high interrupt
		FSR0L_SHADOW		;to save FSR0L during high interrupt
		TmpM			; non-ISR scratch
		TmpT			; non-ISR scratch		
		OutsA			; output port A internal states
		OutsB			; output port B internal states
		OutsC			; output port C internal states
		ENDC

		CBLOCK	0x020	; DO NOT MOVE THIS BLOCK
		Timer1Msb
		Timer1Lsb
		Timer2Msb
		Timer2Lsb
		Timer3Msb
		Timer3Lsb
		Timer4Msb
		Timer4Lsb
		Timer5Msb
		Timer5Lsb
		Timer6Msb
		Timer6Lsb
		Timer7Msb
		Timer7Lsb
		Timer8Msb
		Timer8Lsb
		Timer9Msb
		Timer9Lsb
		Timer10Msb
		Timer10Lsb
		Timer11Msb
		Timer11Lsb
		Timer12Msb
		Timer12Lsb
		Timer13Msb
		Timer13Lsb
		Timer14Msb
		Timer14Lsb
		Timer15Msb
		Timer15Lsb
		Timer16Msb
		Timer16Lsb
		ENDC		
		
		CBLOCK 0x0F2 	; NO NOT RELOCATE THIS BLOCK
		CmdBuff0RdIndx
		CmdBuff0WrIndx
		CmdBuff0Timer
		CmdBuff0TimerValMSB 
		CmdBuff0TimerValLSB 
		CmdBuff1Timer
		CmdBuff1TimerValMSB 
		CmdBuff1TimerValLSB 
		CmdBuff2Timer
		CmdBuff2TimerValMSB 
		CmdBuff2TimerValLSB 
		CmdBuff3Timer
		CmdBuff3TimerValMSB 
		CmdBuff3TimerValLSB 
		ENDC
; ... and incrementing this last adress wraps it to zero.
; don't change this property!

;----------------------------------------------------------------------------
; end of memory map
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;This code executes when a reset occurs.

		ORG     0x0000		;place code at reset vector

ResetVector:	
		bra	Main		;go to beginning of program

;----------------------------------------------------------------------------
;This code executes when a high priority interrupt occurs.

		ORG	0x0008

HighInt:
		bra	HighIntCode	;go to high priority interrupt routine

;----------------------------------------------------------------------------
;This code executes when a low priority interrupt occurs.

		ORG	0x0018

LowInt:	
		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register
		movff	FSR0H,FSR0H_TEMP	;save FSR0H register
		movff	FSR0L,FSR0L_TEMP	;save FSR0L register

		;test other interrupt flags here

		btfss	PIR1,RCIF	;test for RCIF receive interrupt flag
		bra		LowInt1	;if RCIF is not set, done with test
		btfsc	PIE1,RCIE	;else test if Rx interrupt enabled
		bra		GetData		;if so, go get received data
LowInt1:
		btfss	PIR1,TXIF	;test for TXIF transmit interrupt flag
		bra		LowInt2		;if TXIF is not set, done with test
		btfsc	PIE1,TXIE	;else test if Tx interrupt is enabled
		bra		PutData		;if so, go transmit data

;can do special error handling here - an unexpected interrupt occurred 

LowInt2:
		reset			;error if no valid interrupt so reset

		ORG     0x0250		;place code after midi code

		
;------------------------------------
;End of low priority interrupt routine restores context.

EndLowInt:	
		movff	FSR0L_TEMP,FSR0L	;restore FSR0L register
		movff	FSR0H_TEMP,FSR0H	;restore FSR0H register
		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie

;----------------------------------------------------------------------------
;High priority interrupt routine.

HighIntCode:	
		movff	FSR0H,FSR0H_SHADOW	;save FSR0H register
		movff	FSR0L,FSR0L_SHADOW	;save FSR0L register
		;test other interrupt flags here
		btfss	PIR2,TMR3IF	;test for Timer3 receive interrupt flag
		bra		HighInt1	;if RCIF is not set, done with test
		btfsc	PIE2,TMR3IE	;else test if Timer3 interrupt enabled
		bra		Timer3ISR		;if so, go get received data
;can do special error handling here - an unexpected interrupt occurred 
HighInt1:	
		reset			;error if no valid interrupt so reset


;----------------------------------------------------------------------------
;Main routine checks for for reception of a <CR> and then calls a routine to
; move the data to transmit back.

Main:	
		clrf	LATA	; All outputs low
		clrf	LATB	; All outputs low
		clrf	LATC	; All outputs low
		clrf 	TRISA 	; Config PORTA as all outputs
#ifdef ICD
		movlw	0xE0	
		movwf	TRISB
#else
		clrf 	TRISB 	; Config PORTB as all outputs
#endif
		movlw	0x80	;
		movwf 	TRISC 	; Config PORTC all outputs + tristate for serial rx
		movlw 	0x07	; init adc
		movwf	ADCON1	; init adc
		clrf 	ADCON0  ; poweroff adc
		clrf 	OutsA;
		clrf 	OutsB;
		clrf 	OutsC;

		bcf 	LATC,6  ; Light error LED on powerup
						; Wait a second with ALoop and BLoop
		setf	TmpM
BLoop1:
		movlw	0xff;
ALoop1:
		decfsz	WREG;
		goto	ALoop1
		decfsz	TmpM;
		goto 	BLoop1	
		bsf		LATC,6  ; Been patient enough, dim error LED and start
		
		movlw 	CmdBuff0Timer;
		movwf 	CmdBuff0RdIndx;
		movwf 	CmdBuff0WrIndx;
		lfsr  	FSR0 ,0000h
		lfsr  	FSR1 ,0000h
		rcall 	SetupSerial	;set up serial port and buffers
		rcall 	SetupTimer3

;		movlw	0x90;
;		movwf	MidiByte0
;		movlw	0x0F;
;		movwf	MidiByte1
;		movlw	0x40;
;		movwf	MidiByte2

;testloop:
;		nop
;		call	MidiInNoteOn
;		bra		testloop

MainLoop:	
		call	PollMidiIn;
		bra	MainLoop	;go wait for more data


;----------------------------------------------------------------------------
; Timer3 setup

SetupTimer3:
		movlw 	0x01		
		movwf 	T3CON		; timer3 on, other bits cleared
		bsf		IPR2,1		; timer3 = high priority interrupt
		bsf		PIE2,1		; timer3 interrupt enable
	return		


Timer3ISR:
		bcf		PIR2,TMR3IF	; clear interrupt flag
		movlw 	050h 		;0x7F; 0xA0
		movwf	TMR3L	; preset timer
		setf	TMR3H	; preset timer
		movf 	OutsA,0
		movwf	LATA;	
		movf 	OutsB,0
		movwf	LATB;
		movf 	OutsC,0
		movwf	LATC;
ReadBuffer
		movf  	CmdBuff0RdIndx, W;
		subwf 	CmdBuff0WrIndx, W	; result in w
	    bz	  	Timers;	no new task on cmd queue
		movff 	CmdBuff0RdIndx, FSR0L; 
		clrf	FSR0H
		movlw	0x03;
		addwf	CmdBuff0RdIndx
		bnz  	NoWrap;
		movlw 	CmdBuff0Timer; Wrap to start
		movwf 	CmdBuff0RdIndx;
NoWrap
		movf	INDF0,W;
		call	SetPulse
		movf  	POSTINC0, W; W = timer#
		addlw 	010h;		   10h= half of startaddress of timers
		addwf 	WREG; 	   W = (0x10+timer# )* 2				
		movwf 	FSR1L;
		clrf  	FSR1H;	   FSR1 points to timer values
		movf  	POSTINC0, W; W = timer val msb
		movwf 	POSTINC1;
		movf  	POSTINC0, W; W = timer val lsb
		movwf 	POSTINC1;

;************************************************************
; Timers 
;   !!! change to timer interrupt instead of idle loop !!!
Timers
Timer1
		decf 	Timer1Lsb;
		bc		Timer2	;
		decf	Timer1Msb;
		bc		Timer2	;
		setf	Timer1Lsb;
		bcf		Timer1Out
Timer2
		decf 	Timer2Lsb;
		bc		Timer3	;
		decf	Timer2Msb;
		bc		Timer3	;
		setf	Timer2Lsb;
		bcf		Timer2Out
Timer3
		decf 	Timer3Lsb;
		bc		Timer4	;
		decf	Timer3Msb;
		bc		Timer4	;
		setf	Timer3Lsb;
		bcf		Timer3Out
Timer4
		decf 	Timer4Lsb;
		bc		Timer5	;
		decf	Timer4Msb;
		bc		Timer5	;
		setf	Timer4Lsb;
		bcf		Timer4Out
Timer5
		decf 	Timer5Lsb;
		bc		Timer6	;
		decf	Timer5Msb;
		bc		Timer6	;
		setf	Timer5Lsb;
		bcf		Timer5Out
Timer6
		decf 	Timer6Lsb;
		bc		Timer7	;
		decf	Timer6Msb;
		bc		Timer7	;
		setf	Timer6Lsb;
		bcf		Timer6Out
Timer7
		decf 	Timer7Lsb;
		bc		Timer8	;
		decf	Timer7Msb;
		bc		Timer8	;
		setf	Timer7Lsb;
		bcf		Timer7Out
Timer8
		decf 	Timer8Lsb;
		bc		Timer9	;
		decf	Timer8Msb;
		bc		Timer9	;
		setf	Timer8Lsb;
		bcf		Timer8Out
Timer9
		decf 	Timer9Lsb;
		bc		Timer10	;
		decf	Timer9Msb;
		bc		Timer10	;
		setf	Timer9Lsb;
		bcf		Timer9Out
Timer10
		decf 	Timer10Lsb;
		bc		Timer11	;
		decf	Timer10Msb;
		bc		Timer11	;
		setf	Timer10Lsb;
		bcf		Timer10Out
Timer11
		decf 	Timer11Lsb;
		bc		Timer12	;
		decf	Timer11Msb;
		bc		Timer12	;
		setf	Timer11Lsb;
		bcf		Timer11Out
Timer12
		decf 	Timer12Lsb;
		bc		Timer13	;
		decf	Timer12Msb;
		bc		Timer13	;
		setf	Timer12Lsb;
		bcf		Timer12Out
Timer13
		decf 	Timer13Lsb;
		bc		Timer14	;
		decf	Timer13Msb;
		bc		Timer14	;
		setf	Timer13Lsb;
		bcf		Timer13Out
Timer14
		decf 	Timer14Lsb;
		bc		Timer15	;
		decf	Timer14Msb;
		bc		Timer15	;
		setf	Timer14Lsb;
		bcf		Timer14Out
Timer15
		decf 	Timer15Lsb;
		bc		Timer16	;
		decf	Timer15Msb;
		bc		Timer16	;
		setf	Timer15Lsb;
		bcf		Timer15Out
Timer16
		decf 	Timer16Lsb;
		bc		Timer17	;
		decf	Timer16Msb;
		bc		Timer17	;
		setf	Timer16Lsb;
		bcf		Timer16Out
Timer17
	bra	EndHighInt	; resume



MidiInNoteOn: 
;	return
; lookup timer #, read from table in PM:0Fvv
		movf	MidiByte2,W;
		bz		MidiInNoteOffNoVelo;		; velocity=0 -> NOTE OFF
; check if note is in range
		movf	MidiByte2,0;
		bz		MidiInNoteOff;	; velocity=0 -> NOTE OFF
		movlw	0Fh;
		movwf	TBLPTRH
		MOVFF 	MidiByte1,TBLPTRL
		TBLRD* 	; read into TABLAT 
		movf	TABLAT,0
		bz 	MidiInNoteOn2 ;
		decf	TABLAT,0
		movwf	TmpT	; TmpT = timer#
; add to cmd queue	
		clrf  	FSR0H
		movff 	CmdBuff0WrIndx, FSR0L;
		movf	TmpT,W
		movwf 	POSTINC0; write timer #
; lookup velo -> timer val
		movf	MidiByte2,W;	
		addwf	MidiByte2,W; 
		movwf	TBLPTRL;
		movf	TmpT,0; 
		addlw	010h;
		movwf	TBLPTRH
		TBLRD*+	;		
		movff	TABLAT,TmpM
		TBLRD*+	;		
		movf	TABLAT,W
		movwf 	POSTINC0 ; write timer lsb
		movf	TmpM,W;
		movwf 	POSTINC0 ; write timer msb
		movf	FSR0L,W;
		bz		NoWrap2;   corrected: was: bnz
		movwf	CmdBuff0WrIndx;
	return
NoWrap2
		movlw CmdBuff0Timer; Wrap to start
		movwf CmdBuff0WrIndx;
	return


MidiInNoteOn2:
	return


MidiInNoteOffNoVelo:
MidiInNoteOff:
	return

MidiInNoteOff2:
	return

MidiInCtrlChange:
		movlw	0x7B; 			AllNotesOff
		subwf	MidiByte1,0;
		bz		MidiInAllNotesOff;
	return

MidiInAllNotesOff:
		clrf	OutsA
		clrf	OutsB
		clrf	OutsC
		clrf	LATA
		clrf	LATB
		clrf	LATC
	return

	ORG	0x0500
; lookup using a computed goto 
; cfr. PIC18fxx2 datasheet section 4.4
; possibly msb-lsb pc strangeness if this section would cross a code page
SetPulse:
		mullw	0x04;
		movf	PCL,W
		movf	PRODL,W
		addwf	PCL
_sp1
		bsf		Timer1Out
		return
_sp2
		bsf		Timer2Out
		return
_sp3
		bsf		Timer3Out
		return
_sp4
		bsf		Timer4Out
		return
_sp5
		bsf		Timer5Out
		return
_sp6
		bsf		Timer6Out
		return
_sp7
		bsf		Timer7Out
		return
_sp8
		bsf		Timer8Out
		return
_sp9
		bsf		Timer9Out
		return
_sp10
		bsf		Timer10Out
		return
_sp11
		bsf		Timer11Out
		return
_sp12
		bsf		Timer12Out
		return
_sp13
		bsf		Timer13Out
		return
_sp14
		bsf		Timer14Out
		return
_sp15
		bsf		Timer15Out
		return
_sp16
		bsf		Timer16Out
		return
